import os
import json

# nets = {}
# base_dir = os.path.dirname(__file__)
# files = os.listdir(base_dir)
# files = list(filter(lambda x: ".json" in x))
# for file in files:
#     cfg = json.load(open(os.path.join(base_dir, file)))
